import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.bytecode.*;

import java.io.IOException;
import java.io.InputStream;

public class Main {

    public static void main(String[] args) throws Exception {
        ClassPool classPool = ClassPool.getDefault();
        CtClass ctClass = getTestClass(classPool);
        ClassFile classFile = ctClass.getClassFile();

        // The method we want to transform
        CtMethod methodToTransform = ctClass.getMethod("method", "(Ljava/lang/String;)V");

        // The new code "super.method(arg0);"
        String superClassname = classFile.getSuperclass();
        ConstPool constPool = ctClass.getClassFile().getConstPool();
        Bytecode newCode = new Bytecode(constPool);
        newCode.addAload(0); // Loads 'this'
        newCode.addAload(1); // Loads method param 1 (String)
        newCode.addInvokespecial(superClassname, "method", "(Ljava/lang/String;)V");

        // Update max stack and locals
        CodeAttribute currentCodeAttribute = methodToTransform.getMethodInfo().getCodeAttribute();
        currentCodeAttribute.setMaxStack(
                Math.max(currentCodeAttribute.getMaxStack(), newCode.toCodeAttribute().getMaxStack()));
        currentCodeAttribute.setMaxLocals(
                Math.max(currentCodeAttribute.getMaxLocals(), newCode.toCodeAttribute().getMaxLocals()));
        // Insert new code at the beginning (this should update exception table and stack map table offsets)
        CodeIterator codeIterator = currentCodeAttribute.iterator();
        codeIterator.insertEx(newCode.get());

        // I don't want to add this method because it might try to get fix types and query the class pool which does
        // not have the full classpath.
        // methodToTransform.getMethodInfo().rebuildStackMap(classPool);

        ctClass.writeFile();
    }

    private static CtClass getTestClass(ClassPool classPool) {
        try (InputStream in = Main.class.getResourceAsStream("Foo.class")) {
            return classPool.makeClass(in);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
